//BjroBot

#include <Servo.h>
#include <AFMotor.h>

//defineing motor and servos
Servo myservo;
AF_DCMotor motorR(1);
AF_DCMotor motorL(4);

//LED pins
int BL1 = A5;
int BL2 = A4;
int BL3 = A3;
int BL4 = A2;

//global variables
int GoTime = 500;
int TurnTime = 750;

//sensor pins
int irsensor = A0;

void setup() {
  
  Serial.begin(9600);
  Serial.println("BjroBot");

  //setup blinky lights
  pinMode(BL1, OUTPUT);
  pinMode(BL2, OUTPUT);
  pinMode(BL3, OUTPUT);
  pinMode(BL4, OUTPUT);
  
  //sensor pins
  pinMode(irsensor, INPUT);
  
  // turn on motor
  motorR.setSpeed(255);
  motorL.setSpeed(255);
 
  motorR.run(RELEASE);
  motorL.run(RELEASE);
  
  //the servo
  myservo.attach(9);
}

void loop() {
  uint8_t i; //Don't know why this is here but the motors don't work without it...
  
  int dtime = 100; //setting inital delay time
  
  LookCenter(); //moveing the servo to center for inital position
  delay(dtime);

  int WayToGo = 0; //defineing varriable for if statements later on
  
  WayToGo = LookAround(); //going out to find which way to go and how far

  if (WayToGo == 1) { //if center was selected just go forward
    Forward(GoTime);
  }else if (WayToGo == 2) { //if left was selected turn left and go forward
    Lturn(TurnTime);
    Forward(GoTime);
  }else if (WayToGo == 3) { //if right was selected turn right and go forward
    Lturn(TurnTime);
    Forward(GoTime);
  }
}

void ErrorLights(){ //blinky error lights
  
  int BlinkTime = 10;
  
  for(int i = 0; i < 50; i++){    
    digitalWrite(BL1, HIGH);
    delay(BlinkTime);
    digitalWrite(BL1, LOW);
    digitalWrite(BL2, HIGH);
    delay(BlinkTime);
    digitalWrite(BL2, LOW);
    digitalWrite(BL3, HIGH);
    delay(BlinkTime);
    digitalWrite(BL3, LOW);
    digitalWrite(BL4, HIGH);
    delay(BlinkTime);
    digitalWrite(BL4, LOW);
    }
}

int LookAround(){
  int distL = 0; //var for distance left
  int distR = 0; //var for distance right
  int distC = 0; //var for distance center
  int dtime = 200; //inital delay time
  int GoDirection = 0;
  
  LookRight(); //get distance to the right
  delay(dtime); //wait for the servo to get there
  distR = analogRead(irsensor); //set it to distR
  delay(dtime); //wait for the sensor to read correctly
    
  LookCenter(); //get distance to the right
  delay(dtime); //wait for the servo to get there
  distC = analogRead(irsensor); //set it to distC
  delay(dtime); //wait for the sensor to read correctly
  
  LookLeft(); //get distance to the right
  delay(dtime); //wait for the servo to get there
  distL = analogRead(irsensor); //set it to distL
  delay(dtime); //wait for the sensor to read correctly
  
  LookCenter(); //move servo back to center
  delay(dtime);

  GoDirection = DecideDirection(distC, distR, distL); //find farthest distance
  /* if distC is farthest return 1
     if distL is farthest return 2
     if distR is farthest return 3 */
  
  if (GoDirection == 1) { //if DecideDirection came back with center
    GoTime = TheDistance(distC); //calculates safe time to turn the motors on in that direction
  }else if (GoDirection == 2) { //if DecideDirection came back with left
    GoTime = TheDistance(distL); //calculates safe time to turn the motors on in that direction
  }else if (GoDirection == 3) { //if DecideDirection came back with right
    GoTime = TheDistance(distR); //calculates safe time to turn the motors on in that direction
  }else{ //anything else flash error lights and start over
    Serial.println("LookAround");
    ErrorLights();
    softReset();
  }
  
  return(GoDirection);
}

void Stop(){ //stops all motors and turns off lights
  motorR.run(RELEASE);
  motorL.run(RELEASE);
  digitalWrite(BL1, LOW);
  digitalWrite(BL2, LOW);
  digitalWrite(BL3, LOW);
  digitalWrite(BL4, LOW);
}

void Forward(int dtime){ //moves forward for time given in millisec
  motorR.run(FORWARD);
  motorL.run(FORWARD);
  digitalWrite(BL1, HIGH);
  digitalWrite(BL4, HIGH);
  delay(dtime);
  Stop();
}

void Backward(int dtime){ //moves backward for time given in millisec
  motorL.run(BACKWARD);
  motorR.run(BACKWARD);
  digitalWrite(BL2, HIGH);
  digitalWrite(BL3, HIGH);
  delay(dtime);
  Stop();
}

void Rturn(int dtime){ //turns right for time given in millisec
  motorR.run(FORWARD);
  motorL.run(BACKWARD);
  digitalWrite(BL1, HIGH);
  digitalWrite(BL3, HIGH);
  delay(dtime);
  Stop();
}

void Lturn(int dtime){ //turns left for time given in millisec
  motorL.run(FORWARD);
  motorR.run(BACKWARD);
  digitalWrite(BL2, HIGH);
  digitalWrite(BL4, HIGH);
  delay(dtime);
  Stop();
}

int DecideDirection(int distC, int distR, int distL){
  /* if distC is least return 1
     if distL is least return 2
     if distR is least return 3 */
     
     Serial.print("C = ");
     Serial.println(distC);
     Serial.print("R = ");
     Serial.println(distR);
     Serial.print("L = ");
     Serial.println(distL);
     
 if (distL < distR && distL < distC){ //if left is less than right and center
   return(2);
 }else if (distR < distL && distR < distC){ //if right is less than left and center
   return(3); 
 }else if (distC < distL && distC < distR){ //if center is less than left and right
   return(1); 
 }else{ //anything else flash and restart
   Serial.println("DecideDirection");
   ErrorLights();
   softReset();
 }
}

void LookCenter() { //moves the servo to center
  myservo.write(95); //90 is supposed to be center on a servo but mine isn't on exactly straight
}

void LookLeft(){ //moves the servo to look left with the IR sensor
  myservo.write(180);
}

void LookRight(){ //moves the servo to look right with the IR sensor
  myservo.write(15); //again not exactly straight
}

int TheDistance(int range){
  int gotime = 500; //set defalt time
  
  if (range <= 100){
    gotime = 2000;
  }else if (range > 100 && range <= 200){ 
    gotime = 1800;
  }else if (range > 200 && range <= 300){
    gotime = 1600;
  }else if (range > 300 && range <= 400){
    gotime = 1400;
  }else if (range > 400 && range <= 500){
    gotime = 1200;
  }else if (range >= 500){ //its too close need to back up and restart 
    Serial.println("TheDistance");
    Backward(gotime);
    softReset(); 
  }
  
  return(gotime);
}

void softReset(){ //function to restart the sketch... only called if an error happens
asm volatile ("  jmp 0");
} 
